unit unitconnector;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Lightning TConnector class.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

interface

uses
  Windows, Controls, StdCtrls, ExtCtrls, SysUtils, Graphics, Classes, Xdom_3_2,
  common;

const
  colorConnectorOff = clGray;
  colorConnectorOn = clWhite;
  colorInputOff = clOlive;
  colorInputOn = clYellow;
  colorOutputOff = clGreen;
  colorOutputOn = clLime;
  colorBorder = clBlack;
  colorHighlight = clRed;

type
  TConnector = class(TPaintBox)
  private
    colorOn:  TColor;
    colorOff: TColor;
    bitmap:   TBitmap;
    caption:  string;
    tickCountPaint: Int64;
    procedure connectorPaint(Sender: TObject);
    procedure connectorStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure connectorEndDrag(Sender, Target: TObject; X, Y: Integer);
    procedure connectorMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure connectorMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure connectorDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
  public
    level: TLevel;
    constructor Create(AOwner: TComponent); reintroduce; virtual;
    destructor Destroy(); override;
    procedure paintIfChanged(); overload;
    procedure paintIfChanged(newColor: TColor; newCaption: string); overload;
    procedure paintIt();
    procedure clearLevel();
    procedure setLevel(level: TLevel);
    procedure disconnect(); virtual; abstract;
    procedure highlight(); virtual; abstract;
    procedure unHighlight(); virtual; abstract;
    procedure showInfo(); virtual;
  end;
  TOutput = class(TConnector)
  private
  public
    number:     Integer;
    nextModule: Pointer;
    nextInput:  Integer;
    constructor Create(AOwner: TComponent); override;
    function isConnected(): Boolean;
    procedure connect(nextModule: Pointer; nextInput: Integer);
    procedure disconnect(); override;
    procedure highlight(); override;
    procedure unHighlight(); override;
    procedure showInfo(); override;
    procedure connectFromXML(xmlNode: TDOMNode; mapModuleId: TStringList);
    function toXML(): string;
    procedure paintLine(color: TColor; mixMode: Integer; useWindow: Boolean);
    procedure paintLineChain(color: TColor);
  end;
  TInput = class(TConnector)
  private
  public
    number: Integer;
    previousOutput: TOutput;
    constructor Create(AOwner: TComponent); override;
    function isConnected(): Boolean;
    procedure disconnect(); override;
    procedure highlight(); override;
    procedure unHighlight(); override;
    procedure showInfo(); override;
  end;

implementation

uses
  unitlightningsettings, unitformlightning, unitdesktopview, unitmodule;

constructor TConnector.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle   := ControlStyle + [csOpaque];
  Width          := 17;
  Height         := 17;
  DragMode       := dmAutomatic;
  colorOff       := colorConnectorOff;
  colorOn        := colorConnectorOn;
  tickCountPaint := 0;
  level          := levelMin;
  caption        := '';
  with Canvas do
  begin
    Brush.Color := colorOff;
    Pen.Color   := colorBorder;
    Font.Name   := 'Arial';
    Font.Size   := 7;
    Font.Color  := clBlack;
  end;
  bitmap           := TBitmap.Create();
  bitmap.Width     := Width;
  bitmap.Height    := Height;
  with bitmap.Canvas do
  begin
    Brush.Color := colorOff;
    Pen.Color   := colorBorder;
    Font.Name   := 'Arial';
    Font.Size   := 7;
    Font.Color  := clBlack;
  end;
  OnPaint     := connectorPaint;
  OnStartDrag := connectorStartDrag;
  OnEndDrag   := connectorEndDrag;
  OnMouseUp   := connectorMouseUp;
  OnMouseMove := connectorMouseMove;
  OnDragOver  := connectorDragOver;
end;

destructor TConnector.Destroy();
begin
  // cannot disconnect here because disconnecting will cause the module's
  // inputConnected and outputConnected lists to be updated.
  bitmap.Free();
  inherited Destroy();
end;

procedure TConnector.paintIfChanged();
var
  factor: Double;
  newCaption: string;
  newColor: TColor;
begin
  case lightningSettings.displayValues of
    displayValues512:
      // data values are rounded slightly
      newCaption := IntToStr(Trunc((level + 1) / 10));
    displayValues256:
      // DMX values must be exact
      newCaption := IntToStr(Trunc(level / 20));
    displayValues128:
      // MIDI values must be exact
      newCaption := IntToStr(Trunc(level / 40));
    displayValues100:
      // percentages are rounded slightly
      newCaption := IntToStr(Trunc((level + 1) / 51.2));
  else
    newCaption := '';
  end;
  if lightningSettings.visualiseShades then
  begin
    factor := level / levelMax;
    newColor :=
      RGB(
        Trunc(factor * GetRValue(colorOn)),
        Trunc(factor * GetGValue(colorOn)),
        Trunc(factor * GetBValue(colorOn))
      );
  end
  else if level > enabledBoundary then
    newColor := colorOn
  else
    newColor := colorOff;
  paintIfChanged(newColor, newCaption);
end;

procedure TConnector.paintIfChanged(newColor: TColor; newCaption: string);
begin
  if (newColor <> bitmap.Canvas.Brush.Color)
     or
     (newCaption <> caption) then
  begin
    bitmap.Canvas.Brush.Color := newColor;
    caption := newCaption;
    paintIt();
  end;
end;

procedure TConnector.paintIt();
begin
  // only repaint connector box if Color or caption was changed
  with Canvas do
  begin
    if caption <> '' then
    begin
      bitmap.Canvas.Rectangle(Bounds(0, 0, Width, Height));
      if bitmap.Canvas.Brush.Color < colorOff then
        bitmap.Canvas.Font.Color := colorOff
      else
        bitmap.Canvas.Font.Color := clBlack;
      bitmap.Canvas.TextOut(Width div 2 - bitmap.Canvas.TextWidth(caption) div 2, 2, caption);
      Canvas.Draw(0, 0, bitmap);
    end
    else
    begin
      Brush.Color := bitmap.Canvas.Brush.Color;
      Pen.Color := bitmap.Canvas.Pen.Color;
      Rectangle(Bounds(0, 0, Width, Height));
    end;
  end;
end;

procedure TConnector.connectorPaint(Sender: TObject);
begin
  paintIt();
end;

procedure TConnector.connectorStartDrag(Sender: TObject; var DragObject: TDragObject);
begin
  if lightningSettings.lock then
    CancelDrag()
  else
    unHighlight();
end;

procedure TConnector.connectorEndDrag(Sender, Target: TObject; X, Y: Integer);
var
  bInput: TInput;
  bOutput: TOutput;
  bSourceModule, bDestModule: TModule;
begin
  if (Target is TLabel) then
    if (TLabel(Target).Owner is TInput) then
      Target := TInput(TLabel(Target).Owner)
    else if (TLabel(Target).Owner is TOutput) then
      Target := TOutput(TLabel(Target).Owner);
  if (Sender is TLabel) then
    if (TLabel(Sender).Owner is TOutput) then
      Sender := TOutput(TLabel(Sender).Owner)
    else if (TLabel(Sender).Owner is TInput) then
      Sender := TInput(TLabel(Sender).Owner);
  if ((Target is TInput) and (Sender is TOutput))
     or
     ((Sender is TInput) and (Target is TOutput)) then
  begin
    formLightning.stopTimer();
    try
      if Target is TInput then
      begin
        bInput := TInput(Target);
        bOutput := TOutput(Sender);
      end
      else // if Target is TOutput then
      begin
        bInput := TInput(Sender);
        bOutput := TOutput(Target);
      end;
      bSourceModule := TModule(bOutput.Parent);
      bDestModule := TModule(bInput.Parent);
      bInput.disconnect();
      // prevent circular loops (circular loops would cause stack overflows)
      bDestModule.preventCircle(bSourceModule);
      // connect new input to the output
      bOutput.connect(bDestModule, bInput.number);
    finally
      formLightning.startTimer();
    end;
  end;
end;

procedure TConnector.connectorMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if not(lightningSettings.lock) and (Button = mbRight) then // disconnect module
  begin
    unHighlight();
    disconnect();
  end;
end;

procedure TConnector.connectorMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if not(lightningSettings.lock) then
  begin
    showInfo();
    highlight();
  end;
end;

procedure TConnector.connectorDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if Source is TModule then
  begin
    TDesktopView(TModule(Parent).Parent).OnDragOver(Sender, Source, X, Y, State, Accept);
  end
  else
  begin
    // allow only connections from inputs to outputs and disallow self-connections
    Accept :=
      (
        (
          (Source is TInput)
          and
          (Sender is TOutput)
        )
        or
        (
          (Sender is TInput)
          and
          (Source is TOutput)
        )
      )
      and
      (TModule(TConnector(Source).Parent) <> TModule(TConnector(Sender).Parent));
  end;
end;

procedure TConnector.clearLevel();
begin
  paintIfChanged(colorOff, '');
end;

procedure TConnector.setLevel(level: TLevel);
var
  tickCountNow: Int64;
begin
  Self.level := level;
  if lightningSettings.intervalGraphics = 1 then
    paintIfChanged()
  else
  begin
    tickCountNow := tickCount();
    if (tickCountNow - tickCountPaint) > lightningSettings.intervalGraphics then
    begin
      tickCountPaint := tickCountNow;
      paintIfChanged();
    end;
  end;
end;

procedure TConnector.showInfo();
begin
  formLightning.statusBar.Panels[3].Text := Hint;
  formLightning.statusBar.Panels[4].Text := Trim(TModule(Parent).module^.title);
end;

constructor TOutput.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  colorOff := colorOutputOff;
  colorOn  := colorOutputOn;
  bitmap.Canvas.Brush.Color := colorOff;
  nextModule := nil;
  nextInput  := 0;
end;

function TOutput.isConnected(): Boolean;
begin
  Result := nextModule <> nil;
end;

procedure TOutput.connect(nextModule: Pointer; nextInput: Integer);
begin
  disconnect();
  Self.nextModule := nextModule;
  Self.nextInput := nextInput;
  clearLevel();
  TModule(Parent).refreshOutputConnected();
  if isConnected() then
  begin
    with TModule(nextModule) do
    begin
      input[nextInput].previousOutput := Self;
      input[nextInput].clearLevel();
      refreshInputConnected();
    end;
  end;
end;

procedure TOutput.disconnect();
var
  oldNextModule: TModule;
  oldNextInput: Integer;
begin
  inherited;
  if isConnected() then
  begin
    // remove any unpowered line
    paintLine(GetSysColor(COLOR_BTNFACE), R2_COPYPEN, False);
    oldNextModule := nextModule;
    oldNextInput := nextInput;
    nextModule := nil;
    nextInput := 0;
    oldNextModule.input[oldNextInput].previousOutput := nil;
    oldNextModule.input[oldNextInput].clearLevel();
    oldNextModule.refreshInputConnected();
    clearLevel();
    TModule(Parent).refreshOutputConnected();
    TDesktopView(TModule(Parent).Parent).highlightOutput := nil;
  end;
end;

procedure TOutput.highlight();
var
  desktopView: TDesktopView;
begin
  inherited;
  desktopView := TDesktopView(TModule(Parent).Parent);
  if desktopView.highlightOutput <> Self then
  begin
    if desktopView.highlightOutput <> nil then
      desktopView.highlightOutput.unHighlight();
    if isConnected() and lightningSettings.paintHighlightedLines then
    begin
      desktopView.highlightOutput := Self;
      // remove any unpowered line
      paintLine(GetSysColor(COLOR_BTNFACE), R2_COPYPEN, False);
      // paint highlighted line
      paintLine(colorHighlight, R2_NOTXORPEN, True);
      // change output connector border color
      bitmap.Canvas.Pen.Color := colorHighlight;
      paintIt();
      // change input connector border color
      with TModule(nextModule).input[nextInput] do
      begin
        bitmap.Canvas.Pen.Color := colorHighlight;
        paintIt();
      end;
    end;
  end;
end;

procedure TOutput.unHighlight();
var
  desktopView: TDesktopView;
begin
  inherited;
  desktopView := TDesktopView(TModule(Parent).Parent);
  if desktopView.highlightOutput <> nil then
  begin
    // remove highlighted line
    paintLine(colorHighlight, R2_NOTXORPEN, True);
    // restore output connector border color
    bitmap.Canvas.Pen.Color := colorBorder;
    paintIt();
    if isConnected() then
    begin
      // if the corresponding input still exists,
      // restore input connector border color
      with TModule(nextModule).input[nextInput] do
      begin
        bitmap.Canvas.Pen.Color := colorBorder;
        paintIt();
      end;
    end;
    // repaint unpowered line if required
    if lightningSettings.paintUnpoweredLines then
      paintLine(clGray, R2_COPYPEN, False)
    else
      paintLine(GetSysColor(COLOR_BTNFACE), R2_COPYPEN, False);
    desktopView.highlightOutput := nil;
  end;
end;

procedure TOutput.showInfo();
begin
  inherited;
  formLightning.statusBar.Panels[2].Text := 'out ' + IntToStr(number);
end;

procedure TOutput.connectFromXML(xmlNode: TDOMNode; mapModuleId: TStringList);
var
  xmlAttrNode: TDOMNode;
  targetId: string;
  targetInput: Integer;
  module: TModule;
begin
  targetId := '';
  targetInput := 0;
  xmlAttrNode := xmlNode.Attributes.GetNamedItem('targetid');
  if xmlAttrNode <> nil then
    targetId := xmlAttrNode.TextContent;
  xmlAttrNode := xmlNode.Attributes.GetNamedItem('targetinput');
  if xmlAttrNode <> nil then
    targetInput := StrToIntDef(xmlAttrNode.TextContent, 0);
  if (targetId <> '') and (targetInput > 0) then
  begin
    module := TDesktopView(TModule(Parent).Parent).findModule(targetId, mapModuleId);
    if module <> nil then
    begin
      if targetInput <= module.module^.inputs then
      begin
        connect(module, targetInput);
      end;
      // else the saved file contains a reference to a module's input link that doesn't exist in this version of that module.
      // the problem will be ignored to improve backwards compatibility, for example related to the OUT_DMX module.
    end;
  end;
end;

function TOutput.toXML(): string;
begin
  if isConnected() then
    Result := '<output number="' + IntToStr(number) + '" targetid="' + TModule(nextModule).sceneid + '" targetinput="' + IntToStr(nextInput) + '" />'
  else
    Result := '';
end;

procedure TOutput.paintLine(color: TColor; mixMode: Integer; useWindow: Boolean);
var
  bPoint: TPoint;
  DC:     HDC;
  originalMixMode: Integer;
  originalPen, pen: HPEN;
  desktopView: TDesktopView;
begin
  if isConnected()
     and
     TModule(Parent).Visible
     and
     TModule(nextModule).Visible then
  begin
    desktopView := TDesktopView(TModule(Parent).Parent);
    bPoint := desktopView.ScreenToClient(ClientToScreen(Point(0, 0)));
    if useWindow then
      DC := GetWindowDC(desktopView.Handle)
    else
      DC := GetDC(desktopView.Handle);
    try
      originalMixMode := SetROP2(DC, mixMode);
      try
        pen := CreatePen(PS_SOLID, 2, color);
        try
          originalPen := SelectObject(DC, pen);
          try
            MoveToEx(DC, bPoint.x + 9, bPoint.y + 9, nil);
            bPoint := desktopView.ScreenToClient(TModule(nextModule).input[nextInput].ClientToScreen(Point(0, 0)));
            LineTo(DC, bPoint.x + 9, bPoint.y + 9);
          finally
            SelectObject(DC, originalPen);
          end;
        finally
          DeleteObject(pen);
        end;
      finally
        SetROP2(DC, originalMixMode);
      end;
    finally
      ReleaseDC(desktopView.Handle, DC);
    end;
  end;
end;

procedure TOutput.paintLineChain(color: TColor);
var
  i: Integer;
begin
  if isConnected() then
  begin
    paintLine(color, R2_COPYPEN, False);
    for i := 1 to TModule(nextModule).module^.outputs do
      TModule(nextModule).output[i].paintLineChain(color);
  end;
end;

constructor TInput.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  colorOff := colorInputOff;
  colorOn  := colorInputOn;
  bitmap.Canvas.Brush.Color := colorOff;
end;

function TInput.isConnected(): Boolean;
begin
  Result := previousOutput <> nil;
end;

procedure TInput.disconnect();
var
  bOutput: TOutput;
begin
  inherited;
  // find any output using this input and disconnect it
  bOutput := previousOutput;
  if bOutput <> nil then
    bOutput.disconnect();
end;

procedure TInput.highlight();
begin
  inherited;
  // find any output using this input and highlight it
  if previousOutput <> nil then
    previousOutput.highlight();
end;

procedure TInput.unHighlight();
begin
  inherited;
  // find any output using this input and unHighlight it
  if previousOutput <> nil then
    previousOutput.unHighlight();
end;

procedure TInput.showInfo();
begin
  inherited;
  formLightning.statusBar.Panels[2].Text := 'in ' + IntToStr(number);
end;

end.
