unit unitlightningsettings;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Lightning TLightningSettings class.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

interface

const
  lightningVersion = '0.86';

type
  TDisplayValues =
  (
    displayValuesNone = 0,
    displayValues100 = 100,
    displayValues128 = 128,
    displayValues256 = 256,
    displayValues512 = 512
  );
  TLightningSettings = class(TObject)
  private
    procedure load();
    procedure save();
  public
    interval: Cardinal;
    intervalGraphics: Cardinal;
    launchDMXLink: Boolean;
    closeDMXLink: Boolean;
    launchBDetect: Boolean;
    closeBDetect: Boolean;
    splash: Boolean;
    startMinimized: Boolean;
    systemTray: Boolean;
    alwaysOnTop: Boolean;
    tooltips: Boolean;
    paintUnpoweredLines: Boolean;
    paintHighlightedLines: Boolean;
    showLayer: array[0..9] of Boolean;
    visualiseDataFlow: Boolean;
    visualiseShades: Boolean;
    displayValues: TDisplayValues;
    alwaysFinishFades: Boolean;
    lock: Boolean;
    programManager: Boolean;
    sceneManager: Boolean;
    toolBar: Boolean;
    statusBar: Boolean;
    smallPortQuery: Boolean;
    constructor Create(); reintroduce;
    destructor Destroy(); override;
  end;

var
  lightningSettings: TLightningSettings = nil;

implementation

uses
  Windows, Registry, SysUtils;

constructor TLightningSettings.Create();
var
  i: Integer;
begin
  inherited Create();
  interval              := 10;
  intervalGraphics      := 1;
  launchBDetect         := True;
  closeBDetect          := True;
  launchDMXLink         := True;
  closeDMXLink          := True;
  splash                := True;
  startMinimized        := False;
  systemTray            := False;
  alwaysOnTop           := False;
  tooltips              := False;
  paintUnpoweredLines   := True;
  paintHighlightedLines := True;
  visualiseDataFlow     := True;
  visualiseShades       := False;
  displayValues         := displayValues512;
  showLayer[0]          := True;
  for i := 1 to 9 do
    showLayer[i]        := False;
  alwaysFinishFades     := False;
  lock                  := False;
  programManager        := True;
  sceneManager          := True;
  toolBar               := False;
  statusBar             := True;
  smallPortQuery        := False;
  load();
end;

destructor TLightningSettings.Destroy();
begin
  save();
  inherited Destroy();
end;

procedure TLightningSettings.load();
var
  i: Integer;
  bRegistry: TRegistry;
  compatDisplayDataValues: Boolean;
  compatDisplayDMXValues:  Boolean;
  compatShowHiddenModules: Boolean;
begin
  bRegistry := TRegistry.Create(KEY_READ);
  with bRegistry do
  begin
    try
      RootKey := HKEY_CURRENT_USER;
      if OpenKey('Software\Lightning', False) then
      begin
        try
          if ValueExists('Interval') then                        interval              := ReadInteger('Interval');
          if ValueExists('IntervalGraphics') then                intervalGraphics      := ReadInteger('IntervalGraphics');
          if ValueExists('LaunchBDetect') then                   launchBDetect         := ReadBool('LaunchBDetect');
          if ValueExists('CloseBDetect') then                    closeBDetect          := ReadBool('CloseBDetect');
          if ValueExists('LaunchDMXLink') then                   launchDMXLink         := ReadBool('LaunchDMXLink');
          if ValueExists('CloseDMXLink') then                    closeDMXLink          := ReadBool('CloseDMXLink');
          if ValueExists('Splash') then                          splash                := ReadBool('Splash');
          if ValueExists('StartMinimized') then                  startMinimized        := ReadBool('StartMinimized');
          if ValueExists('MinimizeToSystemTray') then            systemTray            := ReadBool('MinimizeToSystemTray');
          if ValueExists('AlwaysOnTop') then                     alwaysOnTop           := ReadBool('AlwaysOnTop');
          if ValueExists('ShowTooltips') then                    tooltips              := ReadBool('ShowTooltips');
          if ValueExists('PaintUnpoweredLine') then              paintUnpoweredLines   := ReadBool('PaintUnpoweredLines');
          if ValueExists('PaintHighlightedLines') then           paintHighlightedLines := ReadBool('PaintHighlightedLines');
          if ValueExists('VisualiseDataFlow') then               visualiseDataFlow     := ReadBool('VisualiseDataFlow');
          if ValueExists('VisualiseUsingAllPossibleShades') then visualiseShades       := ReadBool('VisualiseUsingAllPossibleShades');
          if ValueExists('DisplayValues') then                   displayValues         := TDisplayValues(ReadInteger('DisplayValues'));
          if ValueExists('AlwaysFinishFades') then               alwaysFinishFades     := ReadBool('AlwaysFinishFades');
          if ValueExists('Lock') then                            lock                  := ReadBool('Lock');
          if ValueExists('ProgramManager') then                  programManager        := ReadBool('ProgramManager');
          if ValueExists('SceneManager') then                    sceneManager          := ReadBool('SceneManager');
          if ValueExists('ToolBar') then                         toolBar               := ReadBool('ToolBar');
          if ValueExists('StatusBar') then                       statusBar             := ReadBool('StatusBar');
          if ValueExists('SmallPortQuery') then                  smallPortQuery        := ReadBool('SmallPortQuery');
          for i := 0 to 9 do
            if ValueExists('ShowLayer' + IntToStr(i)) then
              showLayer[i] := ReadBool('ShowLayer' + IntToStr(i));
        finally
          CloseKey();
        end;
        // reopen key with KEY_SET_VALUE privileges so that values can be deleted
        Access := KEY_QUERY_VALUE or KEY_SET_VALUE;
        RootKey := HKEY_CURRENT_USER;
        if OpenKey('Software\Lightning', False) then
        begin
          try
            // backward compatibility
            if ValueExists('DisplayDataValues') then
            begin
              compatDisplayDataValues := ReadBool('DisplayDataValues');
              if DeleteValue('DisplayDataValues') and compatDisplayDataValues then
                displayValues := displayValues512;
            end;
            if ValueExists('DisplayDMXValues') then
            begin
              compatDisplayDMXValues := ReadBool('DisplayDMXValues');
              if DeleteValue('DisplayDMXValues') and compatDisplayDMXValues then
                displayValues := displayValues256;
            end;
            if ValueExists('ShowHiddenModules') then
            begin
              compatShowHiddenModules := ReadBool('ShowHiddenModules');
              if DeleteValue('ShowHiddenModules') then
                showLayer[1] := compatShowHiddenModules;
            end;
            if ValueExists('PaintPoweredLines') then
              DeleteValue('PaintPoweredLines');
          finally
            CloseKey();
          end;
        end;
      end;
    finally
      Free();
    end;
  end;
end;

procedure TLightningSettings.save();
var
  i: Integer;
  bRegistry: TRegistry;
begin
  bRegistry := TRegistry.Create(KEY_WRITE);
  with bRegistry do
  begin
    try
      OpenKey('Software\Lightning', True);
      try
        WriteInteger('Interval',                     interval);
        WriteInteger('IntervalGraphics',             intervalGraphics);
        WriteBool('LaunchBDetect',                   launchBDetect);
        WriteBool('CloseBDetect',                    closeBDetect);
        WriteBool('LaunchDMXLink',                   launchDMXLink);
        WriteBool('CloseDMXLink',                    closeDMXLink);
        WriteBool('Splash',                          splash);
        WriteBool('StartMinimized',                  startMinimized);
        WriteBool('MinimizeToSystemTray',            systemTray);
        WriteBool('AlwaysOnTop',                     alwaysOnTop);
        WriteBool('ShowTooltips',                    tooltips);
        WriteBool('PaintUnpoweredLines',             paintUnpoweredLines);
        WriteBool('PaintHighlightedLines',           paintHighlightedLines);
        WriteBool('VisualiseDataFlow',               visualiseDataFlow);
        WriteBool('VisualiseUsingAllPossibleShades', visualiseShades);
        WriteInteger('DisplayValues',                Ord(displayValues));
        WriteBool('AlwaysFinishFades',               alwaysFinishFades);
        WriteBool('Lock',                            lock);
        WriteBool('ProgramManager',                  programManager);
        WriteBool('SceneManager',                    sceneManager);
        WriteBool('ToolBar',                         toolBar);
        WriteBool('StatusBar',                       statusBar);
        WriteBool('SmallPortQuery',                  smallPortQuery);
        for i := 0 to 9 do
          WriteBool('ShowLayer' + IntToStr(i), showLayer[i]);
      finally
        CloseKey();
      end;
    finally
      Free();
    end;
  end;
end;

end.
